@@ -14,6 +14,7 @@ from io import BytesIO
 from PIL import Image
 from fuzzywuzzy import fuzz
 import nltk
+from multiprocessing import Process, Manager, cpu_count
 
 from wj_social_net_queries.core.instagram import Instagram
 from wj_social_net_queries.core.twitter import Twitter
@@ -38,8 +39,8 @@ IG_TERMS = TERMS["instagram"]
 
 class MainETL:
     """
-    :Date: 2022-10-25
-    :Version: 1.1
+    :Date: 2022-10-31
+    :Version: 1.2.1
     :Author: Joan Felipe Mendoza - Whale & Jaguar Consultants S.A.S.
     :Description: PIPELINE SOBRE AWS
     """
@@ -55,7 +56,6 @@ class MainETL:
             print("NLTK packages already downloaded")
             pass
 
-
     def execute_ig(self, terms=IG_TERMS, locations=LOCATIONS, s3_raw_path="raw", s3_dest_path="processed"):
         """
         :Description: ejecutar pipeline sobre AWS
@@ -89,21 +89,23 @@ class MainETL:
             users = true_cases[["owner_id", "owner_username"]].drop_duplicates().to_dict("records")
         return users
 
-    def execute_tw(self, terms=TW_TERMS, max_tweets=MAX_TWEETS_PER_TERM):
+    def execute_tw(self, terms=TW_TERMS, max_tweets=MAX_TWEETS_PER_TERM, batch=240):
         # Descargar datos desde Twitter y almacenar en DB
         tweets_data, _ = IngestData.twitter(terms=terms, max_tweets=max_tweets, update_db=True)
-        users = {}
-        if len(tweets_data):
-            # Derivar
-            data = self.derive_tw(tweets_data)
-            # Guardar en DB
-            _ = MySQLtools(table="tweets").update_table(data, KEYS["tweets"], batch=300)
-            # Calcular lista de usuarios a contactar
-            true_cases = data[
-                (data[TARGET] == 1) & (data["gender"].isin(["M", "F"])) & (data["country_iso2"].isin(COUNTRIES))
-                ]
-            users = true_cases[["screen_name", "twitter_id", "user_name"]].drop_duplicates().to_dict("records")
-        return users
+        # Calcular lista de usuarios a contactar
+        query = f"SELECT tweet_id, screen_name, twitter_id, created_at, `text`, place, lang, media_url_https, " \
+                f"user_description, user_name, user_location, verified FROM tweets WHERE target IS NULL LIMIT {batch}"
+        while len(tweets_data) > 0:
+            start = datetime.now()
+            # Simular descarga de datos desde streaming
+            tweets_data = MySQLtools().read_mysql_df(query=query)
+            # Derivar datos
+            tweets_data, contacts = self.process_batch_tw(tweets_data)
+            # Almacenar datos en MySQL
+            _ = MySQLtools(table="tweets").update_table(tweets_data, KEYS["tweets"], batch=batch)
+            # Enviar mensajes directos a usuarios
+            print(Responding().twitter_dm(contacts))
+            print(f"Tiempo de proceso: {datetime.now() - start}")
 
     def process_batch_tw(self, tweets_data):
         """
@@ -122,6 +124,7 @@ class MainETL:
             # Calcular lista de usuarios a contactar
             true_cases = data[
                 (data[TARGET] == 1) & (data["gender"].isin(["M", "F"])) & (data["country_iso2"].isin(COUNTRIES))
+                & (data["verified"] == 0)
                 ]
             contacts = true_cases[["screen_name", "twitter_id", "user_name"]].drop_duplicates().to_dict("records")
         return data, contacts
@@ -129,41 +132,7 @@ class MainETL:
     def derive_tw(self, tweets_data, model_type="bert"):
         # Quitar duplicados
         data = pd.DataFrame(tweets_data).drop_duplicates(subset=["tweet_id"]).to_dict('records')
-        data_from_tw = []
-        # extraer información de los datos crudos que vienen desde Twitter
-        for json_data in data:
-            date = str(json_data["created_at"])[:10]
-            screen_name, full_name, description = None, None, None
-            if "screen_name" in json_data:
-                screen_name = json_data["screen_name"]
-            if "user_name" in json_data:
-                full_name = json_data["user_name"]
-            if "user_description" in json_data:
-                description = json_data["user_description"]
-            text = str(json_data["text"])
-            clean_text = NlpUtils().clean_text(text)
-            loc_fields = {"text": True, "place": True, "user_location": True}
-            locations, city, country = self.location_from_text(json_data, loc_fields)
-            brands = self.terms_detection(text, BRANDS["twitter"])
-            keywords = self.terms_detection(text, TERMS["twitter"])
-            kwrds = brands + keywords
-            top_keyword = kwrds[0] if kwrds else None
-            if len(brands) > 0 and country is not None:
-                country = "CO"
-            gender = GenderByText(screen_name, full_name, description, text).predict()
-            data = {
-                **json_data,
-                "date": date,
-                "clean_text": clean_text,
-                "locations": locations,
-                "city": city,
-                "country_iso2": country,
-                "gender": gender,
-                "brands": brands,
-                "keywords": keywords,
-                "top_keyword": top_keyword,
-            }
-            data_from_tw.append(data)
+        data_from_tw = self.multiprocess(target_method=self.transform_data, data=data)
         if data_from_tw:
             df = pd.DataFrame(data_from_tw)
             # Clasificar conversaciones por texto
@@ -176,13 +145,11 @@ class MainETL:
                 # Clasificar conversaciones por imágenes (sólo las que no dieron positivas por texto)
                 obj_img, top_labels, color_img, img_url, target_img = {}, {}, {}, None, 0
                 if row["target_text"] == 0 and row["country_iso2"] in COUNTRIES:
-                    urls = row["media_url_https"] if row["media_url_https"] else []
+                    urls = row["media_url_https"][:2] if row["media_url_https"] else []
                     for url in urls:
                         tw_img = ClassifyImg(url)
-                        obj_img = tw_img.objects
-                        top_labels = tw_img.top_labels
                         color_img = tw_img.colors
-                        target_img = tw_img.predict()
+                        target_img, obj_img, top_labels = tw_img.predict()
                         img_url = url
                         if target_img:
                             break
@@ -201,6 +168,64 @@ class MainETL:
             df = pd.DataFrame()
         return df
 
+    @staticmethod
+    def multiprocess(target_method, data):
+        output = []
+        process_manager = Manager()
+        return_data = process_manager.dict()
+        print(f"Number of CPUs :: {cpu_count()}")
+        cpu_number = cpu_count()
+        jobs = []
+        divide_array = np.array_split(data, cpu_number)
+        for i in range(len(divide_array)):
+            p = Process(target=target_method, args=(divide_array[i], return_data, i))
+            jobs.append(p)
+            p.start()
+        for process in jobs:
+            process.join()
+        for array in return_data.values():
+            output += array
+        return output
+
+    def transform_data(self, data, return_data, index):
+        print(f"DATA Size :: {len(data)}")
+        data_from_tw = []
+        for json_data in data:
+            date = str(json_data["created_at"])[:10]
+            screen_name, full_name, description = None, None, None
+            if "screen_name" in json_data:
+                screen_name = json_data["screen_name"]
+            if "user_name" in json_data:
+                full_name = json_data["user_name"]
+            if "user_description" in json_data:
+                description = json_data["user_description"]
+            text = str(json_data["text"])
+            clean_text = NlpUtils().clean_text(text)
+            loc_fields = {"text": True, "place": True, "user_location": True}
+            locations, city, country = self.location_from_text(json_data, loc_fields)
+            brands = self.terms_detection(text, BRANDS["twitter"])
+            keywords = self.terms_detection(text, TERMS["twitter"])
+            kwrds = brands + keywords
+            top_keyword = kwrds[0] if kwrds else None
+            if len(brands) > 0 and country is None:
+                country = "CO"
+            gender = GenderByText(screen_name, full_name, description, text).predict()
+            data = {
+                **json_data,
+                "date": date,
+                "clean_text": clean_text,
+                "locations": locations,
+                "city": city,
+                "country_iso2": country,
+                "gender": gender,
+                "brands": brands,
+                "keywords": keywords,
+                "top_keyword": top_keyword,
+            }
+            data_from_tw.append(data)
+        key = "process_" + str(index)
+        return_data[key] = data_from_tw
+
     def derive_ig_from_S3(self, folder="raw"):
         """
         :Description: ejecutar proceso de detivación de variables conectándose a AWS S3
@@ -308,6 +333,7 @@ class MainETL:
                         f_name = filename.replace(f"{folder}/", "")
                     image = Image.open(image)
                     ig_img = ClassifyImg(image)
+                    target_img, obj_img, top_labels = ig_img.predict()
                     data = {
                         "filename": f_name,
                         "id": post_id,
@@ -331,13 +357,13 @@ class MainETL:
                         "admin2": admin2,
                         "country_iso2": country_iso2,
                         "gender": gender,
-                        "obj_img": ig_img.objects,
-                        "top_labels": ig_img.top_labels,
+                        "obj_img": obj_img,
+                        "top_labels": top_labels,
                         "color_img": ig_img.colors,
                         "brands": brands,
                         "keywords": keywords,
                         "top_keyword": top_keyword,
-                        TARGET: ig_img.predict(),
+                        TARGET: target_img,
                     }
                     data = {field: data[field] if field in data else None for field in IMG_DB_FIELDS}
                     data_from_images.append(data)
@@ -385,12 +411,8 @@ class MainETL:
 
     @staticmethod
     def terms_detection(text, terms):
-        mentions = []
-        for term in terms:
-            doc = NlpUtils.norm_text(str(text).lower())
-            if term.lower() in doc:
-                mentions.append(term)
-        return mentions
+        doc = NlpUtils.norm_text(str(text).lower())
+        return [t for t in terms if t.lower() in doc]
 
 
 class IngestData:
@@ -559,6 +581,6 @@ class Responding:
                     )
             # Cargar a base de datos
             data = pd.DataFrame(data)
-            MySQLtools(table="report_tw").update_table(data, KEYS["report_tw"], batch=100)
-            users_sent = list(data[data["sent"] == 1]["twitter_id"])
-        return users_sent
+            #MySQLtools(table="report_tw").update_table(data, KEYS["report_tw"], batch=100)
+            users_sent = list(data[data["sent"] == 1]["screen_name"])
+        return [users_sent, data]
